==> sleet-1.0.1-py3-none-any.whl: sleet/base.py <==
"""
This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""

import hashlib
import json
import logging
from datetime import datetime
from typing import Any, Optional
from urllib import parse

from urllib3 import PoolManager
from urllib3.exceptions import HTTPError

from sleet.constants import API_VALID_REGIONS, DEFAULT_CLIENT_HEADERS
from sleet.exceptions import APIStatusError, RegionError, SleetError
from sleet.oauth import OAuth

logger = logging.getLogger(__name__)


class BaseAPI:
    hashing_enabled: bool = False

    def __init__(self, *, region: str, client_id: str, client_secret: str):
        self._api_region = region.lower()
        if self._api_region not in API_VALID_REGIONS:
            raise RegionError(self._api_region)

        self._base_api_url = f"https://{self._api_region}.api.blizzard.com".lower()
        self._client: PoolManager = PoolManager(headers=DEFAULT_CLIENT_HEADERS)
        self._oauth = OAuth(self._api_region, client_id, client_secret)

    def _request(
            self,
            resource: str,
            parameters: Optional[dict[str, Any]],
            headers: Optional[dict[str, str]]
    ) -> dict[str, Any]:
        """
        TODO: Fill in.

        :param resource: The resource endpoint to be queried.
        :param parameters: Additional url parameters to be sent.
        :param headers: Addition request headers to be sent (eg. If-Modified-Since).
        :return: The JSON response from Blizzard.
        """
        resource = f"{self._base_api_url}{resource}"
        if parameters:
            resource = f"{resource}?{parse.urlencode(parameters)}"

        headers = (headers or {}) | self._client.headers | {"Authorization": f"Bearer {self._oauth.token}",
                                                            "X-Sleet-Timestamp": f"{int(datetime.utcnow().timestamp())}"}

        logger.debug(f"Requesting {resource}. Headers: {headers}. Parameters: {parameters}.")

        try:
            response = self._client.request("GET", resource, headers=headers)

        except HTTPError as ex:
            raise SleetError(f"Error querying {resource}.") from ex

        if response.status != 200:
            raise APIStatusError(response.status)

        return self._response_unpack(resource, headers, response)

    def _response_unpack(self, url: str, headers: dict[str, str], response: Any) -> dict[str, Any]:
        """
        TODO: Fill in.

        :param url:
        :param headers:
        :param response:
        :return:
        """
        time_taken = datetime.utcnow().timestamp() - int(headers["X-Sleet-Timestamp"])

        metadata = {
            "url": url,
            "headers": {
                "request": headers,
                "response": dict(response.headers)
            },
            "sha256": None,
            "elapsed": round(time_taken, 3)
        }

        if self.hashing_enabled:
            m = hashlib.sha256()
            m.update(response.data)
            metadata["sha256"] = m.hexdigest()

        response = response.data.decode()

        try:
            response = json.loads(response)

        except json.JSONDecodeError as ex:
            logger.error(f"Failed to obtain new token. Exception: {ex}.")
            raise SleetError(f"Failed to obtain new token. Exception: {ex}.")

        response["@metadata"] = metadata

        return response

    @staticmethod
    def _create_modified_since_header(modified_since: datetime) -> dict[str, str]:
        """
        TODO: Fill in.

        :param modified_since:
        :return:
        """
        if not isinstance(modified_since, datetime):
            raise TypeError(f"Supplied modified_since '{modified_since}' is not a valid datetime.")

        return {"If-Modified-Since": modified_since.strftime("%a, %d %b %Y %H:%M:%S GMT")}

==> sleet-1.0.3-py3-none-any.whl: sleet/base.py <==
"""
This Source Code Form is subject to the terms of the Mozilla Public License, v. 2.0.
If a copy of the MPL was not distributed with this file, You can obtain one at https://mozilla.org/MPL/2.0/.
"""

import hashlib
import json
import logging
from datetime import datetime
from typing import Any, Optional
from urllib import parse

from urllib3 import PoolManager
from urllib3.exceptions import HTTPError

from sleet.constants import API_VALID_REGIONS, DEFAULT_CLIENT_HEADERS
from sleet.exceptions import APIStatusError, RegionError, SleetError
from sleet.oauth import OAuth

logger = logging.getLogger(__name__)


class BaseAPI:
    hashing_enabled: bool = False

    def __init__(self, *, api_region: str, api_id: str, api_secret: str):
        self._api_region = api_region.lower()
        if self._api_region not in API_VALID_REGIONS:
            raise RegionError(self._api_region)

        self._base_api_url = f"https://{self._api_region}.api.blizzard.com".lower()
        self._client: PoolManager = PoolManager(headers=DEFAULT_CLIENT_HEADERS)
        self._oauth = OAuth(self._api_region, api_id, api_secret)

    def _request(
            self,
            resource: str,
            parameters: Optional[dict[str, Any]],
            headers: Optional[dict[str, str]]
    ) -> dict[str, Any]:
        """
        :param resource: The resource endpoint to be queried.
        :param parameters: Additional url parameters to be sent.
        :param headers: Addition request headers to be sent (eg. If-Modified-Since).
        :return: The JSON response from Blizzard.
        """
        resource = f"{self._base_api_url}{resource}"
        if parameters:
            resource = f"{resource}?{parse.urlencode(parameters)}"

        headers = (headers or {}) | self._client.headers | {"Authorization": f"Bearer {self._oauth.token}",
                                                            "X-Sleet-Timestamp": f"{int(datetime.utcnow().timestamp())}"}

        logger.debug(f"Requesting {resource}. Headers: {headers}. Parameters: {parameters}.")

        try:
            response = self._client.request("GET", resource, headers=headers)

        except HTTPError as ex:
            raise SleetError(f"Error querying {resource}.") from ex

        if response.status != 200:
            raise APIStatusError(response.status)

        return self._response_unpack(resource, headers, response)

    def _response_unpack(self, url: str, headers: dict[str, str], response: Any) -> dict[str, Any]:
        """
        Decode an Urllib3 response.
        - A @metadata field will be created that contains the request/response headers.
        - The SHA256 hash of the response data (if hashing_enabled is set to True)

        :param url: The fully qualified URL the request was sent to
        :param headers: The additional headers sent in the request
        :param response: The response object from Urllib3
        :return: The API response with added metadata values
        """
        time_taken = datetime.utcnow().timestamp() - int(headers["X-Sleet-Timestamp"])

        metadata = {
            "url": url,
            "headers": {
                "request": headers,
                "response": dict(response.headers)
            },
            "sha256": None,
            "elapsed": round(time_taken, 3)
        }

        if self.hashing_enabled:
            m = hashlib.sha256()
            m.update(response.data)
            metadata["sha256"] = m.hexdigest()

        response = response.data.decode()

        try:
            response = json.loads(response)

        except json.JSONDecodeError as ex:
            logger.error(f"Failed to obtain new token. Exception: {ex}.")
            raise SleetError(f"Failed to obtain new token. Exception: {ex}.")

        response["@metadata"] = metadata

        return response

    @staticmethod
    def _create_modified_since_header(modified_since: datetime) -> dict[str, str]:
        """
        Create an If-Modified-Since HTTP header from a Python datetime object.

        :param modified_since: A Python datetime object
        :return: A valid If-Modified-Since HTTP header
        """
        if not isinstance(modified_since, datetime):
            raise TypeError(f"Supplied modified_since '{modified_since}' is not a valid datetime.")

        return {"If-Modified-Since": modified_since.strftime("%a, %d %b %Y %H:%M:%S GMT")}
